//
//  KlarnaNativePaymentController.m
//  flutter_klarna_inapp_sdk
//
//  Created by ZhangYu on 2021/9/14.
//

#import "KlarnaNativePaymentController.h"
#import <KlarnaMobileSDK/KlarnaMobileSDK.h>
#import <UIKit/UIKit.h>

@implementation KlarnaNativePaymentController {
    KlarnaPaymentView *_paymentView;
    FlutterMethodChannel *_channel;
    NSString *_clientToken;
    NSString *_returnURL;
    NSString *_category;
    NSString *_loadArgs;
    FlutterResult _result;
    BOOL _isInitialized;
}

- (instancetype)initWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args binaryMessenger:(NSObject<FlutterBinaryMessenger> *)messenger {
    self = [super init];
    if (self) {
        NSDictionary *dict = (NSDictionary *)args;
        _clientToken = [dict valueForKey:@"clientToken"];
        _returnURL = [dict valueForKey:@"returnURL"];
        _category = [dict valueForKey:@"paymentMethodCategory"];
        _loadArgs = [[dict valueForKey:@"loadArgs"] stringValue];
        [self createPaymentVie];
        NSString *channelName = [NSString stringWithFormat:@"klarna_native_sdk_%lld", viewId];
        _channel = [FlutterMethodChannel methodChannelWithName:channelName binaryMessenger:messenger];
        __weak typeof(self) weakSelf = self;
        [_channel setMethodCallHandler:^(FlutterMethodCall * _Nonnull call, FlutterResult  _Nonnull result) {
            __strong typeof(self) strongSelf = weakSelf;
            [strongSelf onMethodCall:call result:result];
        }];
    }
    return self;
}

- (void)createPaymentVie {
    _paymentView = [[KlarnaPaymentView alloc] initWithCategory:_category eventListener:self];
    NSURL *url = [NSURL URLWithString:_returnURL];
    [_paymentView initializeWithClientToken:_clientToken returnUrl: url];
}

- (void)onMethodCall:(FlutterMethodCall *)methodCall result:(FlutterResult)result {
    _result = result;
    if (!_isInitialized) {
        result([FlutterError errorWithCode:@"KlarnaNativeSDKError" message:@"KlarnaNativeSDK is not initialized" details:@"Call 'KlarnaNativeSDK.initialize' before this."]);
        return;
    }
    NSString *method = methodCall.method;
    if ([method  isEqual:@"authorizingSession"]) {
        [_paymentView authorizeWithAutoFinalize:true jsonData:@""];
    } else if ([method isEqual:@"reauthorize"]) {
        [_paymentView reauthorizeWithJsonData:nil];
    } else if ([method isEqual:@"finalize"]) {
        [_paymentView finaliseWithJsonData:nil];
    } else if ([method isEqual:@"loadPaymentReview"]) {
        [_paymentView loadPaymentReview];
    } else if ([method isEqual:@"isAvailable"]) {
        result(@([_paymentView isAvalable]));
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)markNotInitialize:(NSString *)message {
    _isInitialized = false;
    [_channel invokeMethod:@"onInitialized" arguments:@{
        @"isSuccess": @(false),
        @"message": message,
    }];
}

- (UIView *)view {
    return _paymentView;
}

- (void)klarnaAuthorizedWithPaymentView:(KlarnaPaymentView * _Nonnull)paymentView approved:(BOOL)approved authToken:(NSString * _Nullable)authToken finalizeRequired:(BOOL)finalizeRequired {
    if (finalizeRequired) {
        [paymentView finaliseWithJsonData:nil];
    }
    _result(authToken);
}

- (void)klarnaFailedInPaymentView:(KlarnaPaymentView * _Nonnull)paymentView withError:(KlarnaPaymentError * _Nonnull)error {
    
}

- (void)klarnaFinalizedWithPaymentView:(KlarnaPaymentView * _Nonnull)paymentView approved:(BOOL)approved authToken:(NSString * _Nullable)authToken {
    if (authToken) {
        _result(authToken);
    } else {
        _result([FlutterError errorWithCode:@"" message:@"Failed to finalize the Session" details:@""]);
    }
}

- (void)klarnaInitializedWithPaymentView:(KlarnaPaymentView * _Nonnull)paymentView {
    [paymentView loadWithJsonData:_loadArgs];
}

- (void)klarnaLoadedPaymentReviewWithPaymentView:(KlarnaPaymentView * _Nonnull)paymentView {
    
}

- (void)klarnaLoadedWithPaymentView:(KlarnaPaymentView * _Nonnull)paymentView {
    _isInitialized = true;
    NSDictionary *dict = @{
        @"isSuccess": @(true),
        @"message": @"Initialization succeeded."
    };
    [_channel invokeMethod:@"onInitialized" arguments:dict];
}

- (void)klarnaReauthorizedWithPaymentView:(KlarnaPaymentView * _Nonnull)paymentView approved:(BOOL)approved authToken:(NSString * _Nullable)authToken {
    if (authToken) {
        _result(authToken);
    } else {
        _result([FlutterError errorWithCode:@"" message:@"Failed to reauthorizing the Session" details:@""]);
    }
}

- (void)klarnaResizedWithPaymentView:(KlarnaPaymentView * _Nonnull)paymentView to:(CGFloat)newHeight {
    
}

@end
