/*
 * File Created: 2021-09-14 14:01:20
 * Author: ZhangYu (devzhangyu@163.com)
 * -----
 * Last Modified: 2021-09-15 17:30:39
 * Modified By: ZhangYu (devzhangyu@163.com>)
 */

import 'package:flutter/services.dart';

enum CallbackType {
  onInitialized,
}

typedef KlarnaNativeCallback = void Function(
    CallbackType type, dynamic arguments);

class KlarnaNativeSDK {
  KlarnaNativeSDK(this.viewId);

  late MethodChannel _channel = MethodChannel('klarna_native_sdk_$viewId');
  EventChannel _eventChannel = EventChannel('klarna_native_sdk_events');

  final int viewId;

  static const String viewType = "klarna_native_payment_view";

  Future<void> initialize(
      {required String clientToken, required String returnUrl}) async {
    return await _channel.invokeMethod(
      'initialize',
      {'client_token': clientToken, 'return_url': returnUrl},
    );
  }

  Future<void> registerEventListener(void Function(String) listener) async {
    await _channel.invokeMethod('registerEventListener');
    _eventChannel
        .receiveBroadcastStream()
        .map<String>((event) => event)
        .listen(listener);
  }

  void setListener(KlarnaNativeCallback callback) {
    _channel.setMethodCallHandler((MethodCall call) async {
      switch (call.method) {
        case 'onInitialized':
          if (call.arguments is Map) {
            if ((call.arguments as Map).containsKey('isSuccess')) {
              final value = call.arguments['isSuccess'];
              call.arguments['isSuccess'] =
                  value == true || value == 1 || value == '1';
            }
          }
          callback(CallbackType.onInitialized, call.arguments);
          break;
      }
    });
  }

  /// return [authToken]
  Future<String> authorizingSession() async {
    return await _channel.invokeMethod('authorizingSession');
  }

  /// return [authToken]
  Future<String> reauthorize() async {
    return await _channel.invokeMethod('reauthorize');
  }

  /// return [authToken]
  Future<String> finalize() async {
    return await _channel.invokeMethod('finalize');
  }

  /// return [showForm]
  Future<bool> loadPaymentReview() async {
    return await _channel.invokeMethod('loadPaymentReview');
  }

  /// return [available]
  Future<bool> isAvailable() async {
    return await _channel.invokeMethod('isAvailable');
  }
}
